
#include <AFMotor.h>  //Import library to control motor shield
#include <PS4BT.h>    //Import library for PS4 controller
#include <usbhub.h>   //Import library for USB host shield
#include <Servo.h>  //Import library to control the servo

// Satisfy the IDE, which needs to see the include statment in the ino too.
#ifdef dobogusinclude
#include <spi4teensy3.h>
#endif
#include <SPI.h>
AF_DCMotor rightBack(1);                          //Create an object to control each motor
AF_DCMotor rightFront(2);
AF_DCMotor leftFront(3);
AF_DCMotor leftBack(4);
Servo servoLook;                                  //Create an object to control the servo

#define trig A0                                   //Assign the ultrasonic sensor pins
#define echo A2
byte maxDist = 150;                               //Maximum sensing distance (Objects further than this distance are ignored)
byte stopDist = 50;                               //Minimum distance from an object to stop in cm
float timeOut = 2*(maxDist+10)/100/340*1000000;   //Maximum time to wait for a return signal

byte motorSpeed = 155;                             //The maximum motor speed
int motorOffset = 10;                             //Factor to account for one side being more powerful
int turnSpeed = 255 ;                               //Amount to add to motor speed when turning




USB Usb;
//USBHub Hub1(&Usb); // Some dongles have a hub inside
BTD Btd(&Usb); // You have to create the Bluetooth Dongle instance like so

/* You can create the instance of the PS4BT class in two ways */
// This will start an inquiry and then pair with the PS4 controller - you only have to do this once
// You will need to hold down the PS and Share button at the same time, the PS4 controller will then start to blink rapidly indicating that it is in pairing mode
PS4BT PS4(&Btd, PAIR);

// After that you can simply create the instance like so and then press the PS button on the device
//PS4BT PS4(&Btd);

bool printAngle, printTouch;
uint8_t oldL2Value, oldR2Value;


void setup() {
  Serial.begin(115200);
#if !defined(__MIPSEL__)
  while (!Serial); // Wait for serial port to connect 
#endif
  if (Usb.Init() == -1) {
    Serial.print(F("\r\nOSC did not start"));
    while (1); // Halt
  }
  Serial.print(F("\r\nPS4 Bluetooth Library Started"));
  
}
 void accelerate()                                 //Function to accelerate the motors from 0 to full speed
{
  for (int i=0; i<motorSpeed; i++)                //Loop from 0 to full speed
  {
    rightBack.setSpeed(i);                        //Set the motors to the current loop speed
    rightFront.setSpeed(i);
    leftFront.setSpeed(i+motorOffset);
    leftBack.setSpeed(i+motorOffset);
    delay(10);
  }
}

void decelerate()                                 //Function to decelerate the motors from full speed to zero
{
  for (int i=motorSpeed; i!=0; i--)               //Loop from full speed to 0
  {
    rightBack.setSpeed(i);                        //Set the motors to the current loop speed
    rightFront.setSpeed(i);
    leftFront.setSpeed(i+motorOffset);
    leftBack.setSpeed(i+motorOffset); 
    delay(10);
  }
}

void moveForward()                                //Set all motors to run forward
{
  rightBack.run(FORWARD);
  rightFront.run(FORWARD);
  leftFront.run(FORWARD);
  leftBack.run(FORWARD);
}

void stopMove()                                   //Set all motors to stop
{
  rightBack.run(RELEASE);
  rightFront.run(RELEASE);
  leftFront.run(RELEASE);
  leftBack.run(RELEASE);
}

void turnLeft(int duration)                                 //Set motors to turn left for the specified duration then stop
{
    duration = 300;

  rightBack.setSpeed(motorSpeed+turnSpeed);                 //Set the motors to the motor speed
  rightFront.setSpeed(motorSpeed+turnSpeed);
  leftFront.setSpeed(motorSpeed+motorOffset+turnSpeed);
  leftBack.setSpeed(motorSpeed+motorOffset+turnSpeed);
  rightBack.run(FORWARD);
  rightFront.run(FORWARD);
  leftFront.run(BACKWARD);
  leftBack.run(BACKWARD);
  delay(duration);
  rightBack.setSpeed(motorSpeed);                           //Set the motors to the motor speed
  rightFront.setSpeed(motorSpeed);
  leftFront.setSpeed(motorSpeed+motorOffset);
  leftBack.setSpeed(motorSpeed+motorOffset);
  rightBack.run(RELEASE);
  rightFront.run(RELEASE);
  leftFront.run(RELEASE);
  leftBack.run(RELEASE);
  
}

void turnRight(int duration)                                //Set motors to turn right for the specified duration then stop
{
  duration = 300;
  rightBack.setSpeed(motorSpeed+turnSpeed);                 //Set the motors to the motor speed
  rightFront.setSpeed(motorSpeed+turnSpeed);
  leftFront.setSpeed(motorSpeed+motorOffset+turnSpeed);
  leftBack.setSpeed(motorSpeed+motorOffset+turnSpeed);
  rightBack.run(BACKWARD);
  rightFront.run(BACKWARD);
  leftFront.run(FORWARD);
  leftBack.run(FORWARD);
  delay(duration);
  rightBack.setSpeed(motorSpeed);                           //Set the motors to the motor speed
  rightFront.setSpeed(motorSpeed);
  leftFront.setSpeed(motorSpeed+motorOffset);
  leftBack.setSpeed(motorSpeed+motorOffset);
  rightBack.run(RELEASE);
  rightFront.run(RELEASE);
  leftFront.run(RELEASE);
  leftBack.run(RELEASE);
}

int getDistance()                                   //Measure the distance to an object
{
  unsigned long pulseTime;                          //Create a variable to store the pulse travel time
  int distance;                                     //Create a variable to store the calculated distance
  digitalWrite(trig, HIGH);                         //Generate a 10 microsecond pulse
  delayMicroseconds(10);
  digitalWrite(trig, LOW);
  pulseTime = pulseIn(echo, HIGH, timeOut);         //Measure the time for the pulse to return
  distance = (float)pulseTime * 340 / 2 / 10000;    //Calculate the object distance based on the pulse time
  return distance;
}

int checkDirection()                                            //Check the left and right directions and decide which way to turn
{
  int distances [2] = {0,0};                                    //Left and right distances
  int turnDir = 1;                                              //Direction to turn, 0 left, 1 reverse, 2 right
  servoLook.write(180);                                         //Turn servo to look left
  delay(500);
  distances [0] = getDistance();                                //Get the left object distance
  servoLook.write(0);                                           //Turn servo to look right
  delay(1000);
  distances [1] = getDistance();                                //Get the right object distance
  if (distances[0]>=200 && distances[1]>=200)                   //If both directions are clear, turn left
    turnDir = 0;
  else if (distances[0]<=stopDist && distances[1]<=stopDist)    //If both directions are blocked, turn around
    turnDir = 1;
  else if (distances[0]>=distances[1])                          //If left has more space, turn left
    turnDir = 0;
  else if (distances[0]<distances[1])                           //If right has more space, turn right
    turnDir = 2;
  return turnDir;
}


void loop() {
  Usb.Task();

  if (PS4.connected()) {
  

    if (PS4.getAnalogButton(L2) || PS4.getAnalogButton(R2)) { // These are the only analog buttons on the PS4 controller
      Serial.print(F("\r\nL2: "));
      Serial.print(PS4.getAnalogButton(L2));                 // To see if button working or not and to see what value it gives
      Serial.print(F("\tR2: "));
      Serial.print(PS4.getAnalogButton(R2));                 // To see if button working or not and to see what value it gives
    }
    if (PS4.getAnalogButton(L2) != oldL2Value){  // Only write value if it's different
      //PS4.setRumbleOn(PS4.getAnalogButton(L2), PS4.getAnalogButton(R2));
         
    rightFront.run(BACKWARD);         //L2 button runs car backwards
     rightBack.run(BACKWARD);  
   leftFront.run(BACKWARD); 
   leftBack.run(BACKWARD);     
   rightFront.setSpeed(oldL2Value);
   rightBack.setSpeed(oldL2Value);
   leftFront.setSpeed(oldL2Value);
   leftBack.setSpeed(oldL2Value);
 
     
    }    else if (PS4.getAnalogButton(R2) != oldR2Value){
   rightFront.run(FORWARD);     //R2 button runs car Forward
   rightBack.run(FORWARD);
   leftFront.run(FORWARD); 
   leftBack.run(FORWARD);     
   rightFront.setSpeed(oldR2Value);
   leftFront.setSpeed(oldR2Value);
   leftBack.setSpeed(oldR2Value);
   rightBack.setSpeed(oldR2Value); 
    }
      
      oldL2Value = PS4.getAnalogButton(L2);
      oldR2Value = PS4.getAnalogButton(R2);
    if (PS4.getButtonClick(PS)) {    // Use PS button to disconnect the controller
      Serial.print(F("\r\nPS"));
      //PS4.disconnect();
    }
    else {
      
        if (PS4.getButtonClick(CIRCLE)) {
        Serial.print(F("\r\nCircle"));
        PS4.setRumbleOn(RumbleHigh);
         rightBack.run(RELEASE);                         //Ensure all motors are stopped
  rightFront.run(RELEASE);
  leftFront.run(RELEASE);
  leftBack.run(RELEASE);
        }
      
      if (PS4.getButtonClick(UP)) {
        Serial.print(F("\r\nUp"));
        PS4.setLed(Red);
        rightFront.run(FORWARD);     
        rightBack.run(FORWARD);
        leftFront.run(FORWARD); 
        leftBack.run(FORWARD);
        rightBack.setSpeed(155);     
        rightFront.setSpeed(150);
        leftFront.setSpeed(210);
        leftBack.setSpeed(210);
    } 
    if (PS4.getButtonClick(RIGHT)) {      //Make car go right
        Serial.print(F("\r\nRight"));
        PS4.setLed(Blue);
        rightFront.run(BACKWARD);     
        rightBack.run(FORWARD);
        leftFront.run(FORWARD); 
        leftBack.run(BACKWARD);
        rightBack.setSpeed(155);     
        rightFront.setSpeed(120);
        leftFront.setSpeed(150);
        leftBack.setSpeed(150);
      }
  if (PS4.getButtonClick(DOWN)) {
        Serial.print(F("\r\nDown"));
        PS4.setLed(Yellow);
        rightFront.run(BACKWARD);     
        rightBack.run(BACKWARD);
        leftFront.run(BACKWARD); 
        leftBack.run(BACKWARD);
        rightBack.setSpeed(155);     
        rightFront.setSpeed(150);
        leftFront.setSpeed(210);
        leftBack.setSpeed(210);
      } 
      if (PS4.getButtonClick(LEFT)) {      //Make car go left
        Serial.print(F("\r\nLeft"));
        PS4.setLed(Green);
        rightFront.run(FORWARD);     
    

   rightBack.run(BACKWARD);
   leftFront.run(BACKWARD); 
   leftBack.run(FORWARD);
   rightBack.setSpeed(155);     
   rightFront.setSpeed(150);
   leftFront.setSpeed(150);
   leftBack.setSpeed(150);
      }

      if (PS4.getButtonClick(L1)){      // Make car go diagnally to the left
        Serial.print(F("\r\nL1"));
        rightFront.run(FORWARD);     
        rightBack.run(RELEASE);
        leftFront.run(RELEASE); 
        leftBack.run(FORWARD);
        rightFront.setSpeed(120);
        leftBack.setSpeed(150);
      }
      if (PS4.getButtonClick(L3)){    // Make car pivot left side
        Serial.print(F("\r\nL3"));
         rightFront.run(FORWARD);     
   rightBack.run(FORWARD);
   leftFront.run(BACKWARD); 
   leftBack.run(BACKWARD);
   rightBack.setSpeed(155);     
   rightFront.setSpeed(120);
   leftFront.setSpeed(150);
   leftBack.setSpeed(150);
      }
      if (PS4.getButtonClick(R1)){    // Make car go diagnally to the right
        Serial.print(F("\r\nR1"));
        rightFront.run(RELEASE);     
   rightBack.run(FORWARD);
   leftFront.run(FORWARD); 
   leftBack.run(RELEASE);
   rightBack.setSpeed(155);     
  
   leftFront.setSpeed(150);
      } 
      if (PS4.getButtonClick(R3)){    // Make car pivot right side
        Serial.print(F("\r\nR3"));
         rightFront.run(BACKWARD);     
   rightBack.run(BACKWARD);
   leftFront.run(FORWARD); 
   leftBack.run(FORWARD);
   rightBack.setSpeed(155);     
   rightFront.setSpeed(120);
   leftFront.setSpeed(150);
   leftBack.setSpeed(150);
      }
if (PS4.getButtonClick(TRIANGLE)) {      //For autonomous mode
        Serial.print(F("\r\nTriangle"));
        PS4.setRumbleOn(RumbleLow);
        servoLook.write(90);                            //Set the servo to look straight ahead
  delay(750);
  int distance = getDistance();                   //Check that there are no objects ahead
  if(distance >= stopDist)                        //If there are no objects within the stopping distance, move forward
  {
    moveForward();
  }
  while(distance >= stopDist)                     //Keep checking the object distance until it is within the minimum stopping distance
  {
    distance = getDistance();
    delay(250);
  }
  stopMove();                                     //Stop the motors
  int turnDir = checkDirection();                 //Check the left and right object distances and get the turning instruction
  Serial.print(turnDir);
  switch (turnDir)                                //Turn left, turn around or turn right depending on the instruction
  {
    case 0:                                       //Turn left
      turnLeft (400);
      break;
    case 1:                                       //Turn around
      turnLeft (700);
      break;
    case 2:                                       //Turn right
      turnRight (400);
      break;
        
      }
     
          }
        }
      }
}

   




 
